/* bcb.h
** 
** This file declares several functions and defines
** several data structures to be used by the runbcb.c
** program.  The function definitions are in bcb.c
**
** Mike Weber
** 3/2000
*/

#ifndef BCBH
#define BCBH

#define MAXDV       100
#define MAXPOPSIZE  2500
#define BIGM        (pow(10.0, 30.0))
#define NUMPOINTS   1000

/* POPULATION DATA STRUCTURES */
struct SOLUTION {
  double dv[MAXDV];                   /* decision variable values   */
  double obj;                         /* objective value            */
  double maxcon;                      /* max constraint value       */
  double avgcon;                      /* average constraint value   */
  double fitness;                     /* fitness of solution        */
  long numdv;                         /* number of design variables */
};

struct POPULATION {
  long popsize;                       /* number of solutions        */
  struct SOLUTION points[MAXPOPSIZE]; /* list of solutions          */
  int sortbyobj[MAXPOPSIZE];          /* indices sorting by obj     */
  int sortbymax[MAXPOPSIZE];          /* indices sorting by max con */  
};


/* ALGORITHM and PROBLEM PARAMETERS */
struct PARAMLIST {
  long popsize;                       /* population size            */
  long numgens;                       /* number of generations      */
  double sigma_m, sigma_r;            /* standard deviation factors */
  double pen1, pen2;                  /* penalty factors            */
  int parchoose;                      /* parent selection method    */
  long numdv;                         /* number of decision vars    */
  double ub[MAXDV];                   /* decision var upper bounds  */
  double lb[MAXDV];                   /*              lower bounds  */
};

double distance(struct SOLUTION p1, struct SOLUTION p2);
void evalcon(struct POPULATION *pop);
void evalobj(struct POPULATION *pop, double data[NUMPOINTS], int model, int numpoints);
void fitness(struct POPULATION *pop, struct SOLUTION *best, 
	     struct PARAMLIST *par);
void fitnesssrt(struct POPULATION *pop);
void initpop(struct POPULATION *pop, struct PARAMLIST *par);
void poolpops(struct POPULATION *pool, struct POPULATION *pop1, 
	      struct POPULATION *pop2);
void read_param(struct PARAMLIST *par);
void recombine(struct POPULATION *newpop, struct POPULATION *oldpop, 
               struct PARAMLIST *par);
void g_select(struct POPULATION *new, struct POPULATION *pooled);
void zbest(struct POPULATION *pop, struct SOLUTION *best);

#endif
